module Gargantext.Components.GraphQL.Endpoints where

import Data.Array as A
import Data.Bifunctor (rmap)
import Data.Either (Either(..), note)
import Data.Map as Map
import Data.Maybe (Maybe(..))
import Data.Tuple (Tuple(..))
import Effect.Class (liftEffect)
import Gargantext.Components.GraphQL (queryGql, mutationGql)
import Gargantext.Components.GraphQL.Contact (AnnuaireContact, annuaireContactQuery)
import Gargantext.Components.GraphQL.Context as GQLCTX
import Gargantext.Components.GraphQL.IMT as GQLIMT
import Gargantext.Components.GraphQL.NLP as GQLNLP
import Gargantext.Components.GraphQL.Node (Corpus, Node, getNodeChildrenQuery, getNodeParentQuery, getNodeByIdQuery, getCorpusByIdQuery, getNodesQuery)
import Gargantext.Components.GraphQL.Team (Team, teamQuery)
import Gargantext.Components.GraphQL.Tree (TreeFirstLevel, getTreeByRootIdQuery, BreadcrumbInfo, breadcrumbQuery)
import Gargantext.Components.GraphQL.User (UserInfo, getUserInfoQuery, User, getUserByIdQuery)
import Gargantext.Components.Lang (Lang)
import Gargantext.Config.REST (RESTError(..), AffRESTError)
import Gargantext.Core.NgramsTable.Types (NgramsTerm(..))
import Gargantext.Prelude
import Gargantext.Routes (AppRoute(..))
import Gargantext.Sessions (Session(..))
import Gargantext.Types (CorpusId, NodeType)
import Gargantext.Utils.GraphQL (unwrapGraphQLResult)
import Gargantext.Utils.Reactix as R2
import GraphQL.Client.Args (onlyArgs, OrArg(..), IgnoreArg(..))
import GraphQL.Client.Variables (withVars)

here :: R2.Here
here = R2.here "Gargantext.Components.GraphQL.Endpoints"

getIMTSchools :: Session -> AffRESTError (Array GQLIMT.School)
getIMTSchools session = do
  eRes <- queryGql session "get imt schools" $ GQLIMT.getSchoolsQuery
  pure $ rmap _.schools eRes

-- liftEffect $ here.log2 "[getIMTSchools] imt_schools" imt_schools
-- pure $ Right imt_schools

getNodeById :: Session -> Int -> AffRESTError Node
getNodeById session nodeId = do
  let params = { id: nodeId }
  eRes <- queryGql session "get node" $ getNodeByIdQuery `withVars` params
  pure $ _.node <$> eRes

getCorpusById :: Session -> Int -> AffRESTError Corpus
getCorpusById session corpusId = do
  let params = { id: corpusId }
  eRes <- queryGql session "get corpus by id" $
    getCorpusByIdQuery `withVars` params
  pure $ _.corpus <$> eRes

getNodeParent :: Session -> Int -> NodeType -> AffRESTError Node
getNodeParent session nodeId parentType = do
  let params = { nodeId, parentType }
  eRes <- queryGql session "get node parent" $
    getNodeParentQuery nodeId parentType
  pure $ unwrapGraphQLResult (Tuple "node parent" params) (_.parent <$> eRes)

getNodeChildren :: Session -> Int -> NodeType -> AffRESTError (Array Node)
getNodeChildren session nodeId childType = do
  let
    params =
      { id: nodeId
      , nodeType: childType
      }
  eRes <- queryGql session "get node child" $
    getNodeChildrenQuery `withVars` params
  -- liftEffect $ here.log2 "[getNodeParent] node_parent" node_parent
  --pure node_parent
  pure $ rmap _.nodes eRes

getUser :: Session -> Int -> AffRESTError User
getUser session id = do
  let params = { id }
  eRes <- queryGql session "get user by id" $ getUserByIdQuery `withVars` params
  pure $ _.user <$> eRes

updateUserPubmedAPIKey :: Session -> Int -> String -> AffRESTError Unit
updateUserPubmedAPIKey session user_id api_key = do
  eRes <- mutationGql session
    "update_user_pubmed_api_key"
    { update_user_pubmed_api_key: onlyArgs
        { user_id
        , api_key
        }
    }
  -- { update_user_pubmed_api_key }
  pure $ rmap (const unit) eRes

updateUserEPOAPIUser :: Session -> Int -> String -> AffRESTError Unit
updateUserEPOAPIUser session user_id api_user = do
  eRes <- mutationGql session
    "update_user_epo_api_user"
    { update_user_epo_api_user: onlyArgs
        { user_id
        , api_user
        }
    }
  pure $ rmap (const unit) eRes

updateUserEPOAPIToken :: Session -> Int -> String -> AffRESTError Unit
updateUserEPOAPIToken session user_id api_token = do
  eRes <- mutationGql session
    "update_user_epo_api_token"
    { update_user_epo_api_token: onlyArgs
        { user_id
        , api_token
        }
    }
  pure $ rmap (const unit) eRes

getUserInfo :: Session -> Int -> AffRESTError UserInfo
getUserInfo session id = do
  eRes <- queryGql session "get user info" $ getUserInfoQuery `withVars` { id }
  pure $ _.userInfo <$> eRes

getAnnuaireContact :: Session -> Int -> AffRESTError AnnuaireContact
getAnnuaireContact session id = do
  let params = { id }
  eRes <- queryGql session "get annuaire contact" $
    annuaireContactQuery `withVars` params
  pure $ _.annuaireContact <$> eRes

getTreeFirstLevel :: Session -> Int -> AffRESTError TreeFirstLevel
getTreeFirstLevel session id = do
  let params = { id }
  eRes <- queryGql session "get tree first level" $
    getTreeByRootIdQuery id
  -- getTreeByRootIdQuery `withVars` params
  pure $ (\{ root, children, parents } -> { root, children, parent: A.head parents }) <$> eRes

-- pure $ (\{ root } -> { root, children: [], parent: Nothing }) <$> eRes

getTeam :: Session -> Int -> AffRESTError Team
getTeam session id = do
  let params = { id }
  eRes <- queryGql session "get team" $ teamQuery `withVars` params
  pure $ _.team <$> eRes

type SharedFolderId = Int
type TeamNodeId = Int

deleteTeamMembership :: Session -> SharedFolderId -> TeamNodeId -> AffRESTError Int
deleteTeamMembership session sharedFolderId teamNodeId = do
  let token = getToken session
  eRes <- mutationGql session
    "delete_team_membership"
    { delete_team_membership: onlyArgs
        { token: token
        , shared_folder_id: sharedFolderId
        , team_node_id: teamNodeId
        }
    }
  pure $ case eRes of
    Left err -> Left err
    Right { delete_team_membership } ->
      case A.head delete_team_membership of
        Nothing -> Left (CustomError $ "Failed  to delete team membership. team node id=" <> show teamNodeId <> " shared folder id=" <> show sharedFolderId)
        Just _ -> Right sharedFolderId
  where
  getToken (Session { token }) = token

getNodeContext :: Session -> Int -> Int -> AffRESTError GQLCTX.NodeContext
getNodeContext session contextId nodeId = do
  let params = { contextId, nodeId }
  eRes <- queryGql session "get context by id and node" $
    GQLCTX.getContextByIdAndNodeQuery `withVars` params
  pure $ _.context <$> eRes

type ContextsForNgramsGQL = { contextsForNgrams :: Array GQLCTX.Context }

getContextsForNgrams :: Session -> CorpusId -> Array String -> Boolean -> AffRESTError (Array GQLCTX.Context)
getContextsForNgrams session corpus_id ngrams_terms logic = do
  let
    params =
      { corpus_id
      , ngrams_terms: GQLCTX.NgramsTerms ngrams_terms
      , and_logic: show logic
      }
    query = GQLCTX.getContextsForNgramsQuery `withVars` params
  eRes <- queryGql session "get contexts for ngrams" query
  pure $ rmap _.contextsForNgrams eRes

updateNodeContextCategory :: Session -> Int -> Int -> Int -> AffRESTError Int
updateNodeContextCategory session context_id node_id category = do
  eRes <- mutationGql session
    "update_node_context_category"
    { update_node_context_category: onlyArgs
        { context_id
        , node_id
        , category
        }
    }
  pure $ case eRes of
    Left err -> Left err
    Right { update_node_context_category } -> case A.head update_node_context_category of
      Nothing -> Left (CustomError $ "Failed to update node category")
      Just _ -> Right context_id

getLanguages :: Session -> AffRESTError (Map.Map Lang GQLNLP.LanguageProperties)
getLanguages session = do
  let query = GQLNLP.nlpQuery
  eRes <- queryGql session "get languages" query
  pure $ (\res -> Map.fromFoldable $ (\{ lt_lang, lt_server } -> Tuple lt_lang lt_server) <$> res.languages) <$> eRes

getNgramsForContextAndList :: Session -> Int -> Int -> AffRESTError (Array NgramsTerm)
getNgramsForContextAndList session contextId listId = do
  let query = GQLCTX.getNgramsForContextAndListQuery `withVars` { contextId, listId }
  eRes <- queryGql session "get ngrams for context and list" query
  pure $ rmap (\{ ngramsForContextAndListId } -> NormNgramsTerm <$> ngramsForContextAndListId) eRes

getBreadcrumb :: Session -> Int -> AffRESTError BreadcrumbInfo
getBreadcrumb session nodeId = do
  eRes <- queryGql session "get breadcrumb branch" $ breadcrumbQuery nodeId
  -- liftEffect $ here.log2 "[getBreadcrumb] breadcrumb" tree_branch
  pure $ eRes
